//==============================================================================
// VARIABLES LOCALES
//==============================================================================

var color = "rgb(0,0,0)", colors; // les couleurs proposes
var canvas, context, canvaso, contexto;
var cleanCanvas; // bouton effacer
var tool_default = 'pencil', toolSelected; // figure choisie
var isPenDown = false; // =true quand doigt sur le canvas
var localPen = {}; // contiendra les coordonnes du doigt touchant le canvas
var toolSize, localLineThickness = 1; // largeur du pinceau

//==============================================================================
// INITIALISATION
//==============================================================================

function startup () {
	initFrame();
	registerInputListeners();	
	window.scrollTo(0,1);
}
window.onresize = resizeHandler;

/* Dfinition des tailles initialement et redimensionnement quand changement d'orientation */
function resizeHandler(){
	canvaso.width=getWidth();
	if(canvaso.width < getHeight()){ // portrait
		canvaso.height=canvaso.offsetWidth;
		/* Positionnement de la barre de couleur centr */
		document.getElementById('colorbar').style.width="300px";
		var colWidth = document.getElementById('colorbar').offsetWidth ;
		document.getElementById('colorbar').style.marginLeft = ((canvaso.width - colWidth)/2)+"px"
		document.getElementById('controls').style.marginLeft = ((canvaso.width - colWidth)/2)+"px"
	}else{ // paysage
		canvaso.height=getHeight()/2;
		document.getElementById('colorbar').style.width="450px";
		document.getElementById('colorbar').style.marginLeft = ((canvaso.width - 450)/2)+"px"
		document.getElementById('controls').style.marginLeft = ((canvaso.width - 450)/2)+"px"
	}
	if(canvas!=null){
		canvas.width  = canvaso.width;
		canvas.height = canvaso.height;
	}
}
/* initialisation du canvas et des variables taille, couleur, forme choisie et bouton effacer */
function initFrame () {  
	canvaso = document.getElementById('imageView');
	resizeHandler();
	
	/* cration d'un deuxieme canvas qui servira a memoriser les figures deja saisies */
	contexto = canvaso.getContext('2d');
	var container = canvaso.parentNode;
	canvas = document.createElement('canvas');
	canvas.id     = 'imageTemp';
	canvas.width  = canvaso.width;
	canvas.height = canvaso.height;
	container.appendChild(canvas);
	context = canvas.getContext('2d');
	context.lineCap = "round";
	
	/* diametres, figures, effacer et couleurs */
	toolSize = document.getElementById("taille")
	toolSize.selectedIndex = 0;
	toolSelected = document.getElementById("dtool")
	toolSelected.selectedIndex = 0;
	cleanCanvas = document.getElementById("cleanCanvas");
	colors = document.getElementsByName("color");
}

/* Listeners avec utilisation de jquery mobile */
function registerInputListeners () {
	// sur le canvas
	canvas.addEventListener("touchstart", touchDownListener, false);
	canvas.addEventListener("touchend", touchUpListener, false);
	canvas.addEventListener("touchmove", touchMoveListener, false);
	
	// sur les listes droulantes tailles et figures
	toolSize.onchange = thicknessSelectListener;
	toolSelected.onchange = toolSelectListener;
	
	// sur le bouton effacer
	$('#cleanCanvas').live('tap',function(event) {
		contexto.clearRect(0, 0, canvas.width, canvas.height);
		context.clearRect(0, 0, canvas.width, canvas.height);
		contexto.drawImage(canvas, 0, 0);
	});
	
	// sur les diffrentes couleurs proposes
	for(var i=0; i<colors.length; i++){
		colors[i].addEventListener("touchstart", selCol, false);
	}
}

/* Permet d'effacer le canvas */
function cleanCanvas(e){
	context.clearRect(0, 0, canvas.width, canvas.height);
	context.drawImage(canvas, 0, 0);
}

/* Permet de garder les figures ralises sur le deuxime canvas */
function img_update () {
	contexto.drawImage(canvas, 0, 0);
	context.clearRect(0, 0, canvas.width, canvas.height);
}

/* Permet de rcuprer la taille slectionne dans le menu droulant */
function thicknessSelectListener (e) {
	var newThickness = this.options[this.selectedIndex].value;
	localLineThickness = getValidThickness(newThickness);
}

/* Permet de rcuprer la figure slectionne dans le menu droulant */
function toolSelectListener (e) {
	tool_default = this.options[this.selectedIndex].value;
}

/* Permet de changer la couleur */
function selCol(e) { 
	color = this.style.backgroundColor;
}

/* Rcuperation de la largeur de l'cran */
function getWidth(){
    xWidth = null;
    if(window.screen != null)
      xWidth = window.screen.availWidth;
 
    if(window.innerWidth != null)
      xWidth = window.innerWidth;
 
    if(document.body != null)
      xWidth = document.body.clientWidth;
 
    return xWidth;
}

/* Rcuperation de la hauteur de l'cran */
function getHeight() {
  xHeight = null;
  if(window.screen != null)
    xHeight = window.screen.availHeight;
 
  if(window.innerHeight != null)
    xHeight =   window.innerHeight;
 
  if(document.body != null)
    xHeight = document.body.clientHeight;
 
  return xHeight;
}

//==============================================================================
// TOUCH-INPUT EVENT LISTENERS
//==============================================================================
function touchDownListener (e) {
	if (event.target.nodeName != "SELECT") {
		e.preventDefault();
	}
	// Rcupration des coordonnes du doigt sur le canvas
	var touchX = e.changedTouches[0].clientX - canvas.offsetLeft;
	var touchY = e.changedTouches[0].clientY - canvas.offsetTop;
	if (!isPenDown) {
		penDown(touchX, touchY);
	}
}

function touchMoveListener (e) {
	e.preventDefault();
	var touchX = e.changedTouches[0].clientX - canvas.offsetLeft;
	var touchY = e.changedTouches[0].clientY - canvas.offsetTop;
	penMove(touchX, touchY);
}

function touchUpListener () {
	penUp();
}

//==============================================================================
// PEN
//==============================================================================
var departX, departY;

// doigt pos
function penDown (x, y) {
	isPenDown = true;
	localPen.x = x;
	localPen.y = y;
	departX = x;
	departY = y;
}

// On dessine en fonction de la figure choisie (doigt en dplacement)
function penMove (x, y) { 
	if (isPenDown) {
		if(tool_default=="pencil"){
			drawLine(color, localLineThickness, localPen.x, localPen.y, x, y);
			localPen.x = x;
			localPen.y = y;
		}else if(tool_default=="line"){
			drawLine(color, localLineThickness, departX, departY, x, y);
			localPen.x = x;
			localPen.y = y;
		}else if(tool_default=="rect"){
			var x1 = Math.min(x,  departX),
			y1 = Math.min(y,  departY),
			w1 = Math.abs(x - departX),
			h1 = Math.abs(y - departY);
			context.clearRect(0, 0, canvas.width, canvas.height);
			context.strokeStyle = color;
			context.strokeRect(x1, y1, w1, h1);
		}else{ // =="circle"
			context.clearRect(0, 0, canvas.width, canvas.height);
			context.beginPath();
			var x2 = Math.min(x,  departX),
			y2 = Math.min(y,  departY),
			w2 = Math.abs(x - departX),
			h2 = Math.abs(y - departY),
			radius = Math.sqrt(w2*w2 + h2*h2);
			context.arc(x2, y2, radius, 0, Math.PI * 2, true);
			context.strokeStyle = color;
		    context.fillStyle = color;
			context.fill();
			context.stroke();
			context.closePath();
		}
	}
}

// doigt lev
function penUp () {
	img_update();
	isPenDown = false;
}

//==============================================================================
// DRAWING
//==============================================================================
function drawLine (color, thickness, x1, y1, x2, y2) {
	context.strokeStyle = color;
	context.lineWidth   = thickness;
	context.beginPath();
	context.moveTo(x1, y1)
	context.lineTo(x2, y2);
	if(tool_default=="line"){
		context.clearRect(0, 0, canvas.width, canvas.height);
	}
	context.stroke();
}

//==============================================================================
// DATA VALIDATION
//==============================================================================
function getValidThickness (value) {
	value = parseInt(value);
	var thickness = isNaN(value) ? defaultLineThickness : value;
	return Math.max(1, thickness);
}
